@@ -19,6 +19,8 @@ import os
 import random
 import string
 import filecmp
+import tempfile
+import zipfile
 
 import six
 import yaml
@@ -312,6 +314,58 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         '''Clean up any test client'''
         rpc('drop', self.portalcik, self.client.rid)
 
+    @attr('auth')
+    def auth_cik_clientid_test(self):
+        '''Test using the <CIK:cRID> auth format'''
+        cik = self.client.parentcik
+        rid = self.client.rid
+
+        auth = '{0}:c{1}'.format(cik, rid)
+        isok = rpc('info', auth)
+        self.ok(isok, 'info with <CIK:cRID>')
+
+    @attr('auth')
+    def auth_cik_resourceid_test(self):
+        '''Test using the <CIK:rRID> auth format'''
+        cik = self.client.parentcik
+        rid = self._createDataports()
+
+        auth = '{0}:r{1}'.format(cik, rid['string'].rid)
+        isok = rpc('info', auth)
+        self.ok(isok, 'info with <CIK:rRID>')
+
+    @attr('auth')
+    def auth_shortcut_clientid_test(self):
+        '''Test using the <SHORTCUT:cRID> auth format'''
+        cik = self.client.parentcik
+        rid = self.client.rid
+
+        with tempfile.NamedTemporaryFile('w+') as cfgfile:
+            cfgfile.write("keys:\n")
+            cfgfile.write("    testme: {0}".format(cik))
+            cfgfile.flush()
+            cfgs = "--config={0}".format(cfgfile.name)
+            auth = 'testme:c{0}'.format(rid)
+            isok = rpc(cfgs, 'info', auth)
+            self.ok(isok, 'info with <SHORTCUT:cRID>')
+
+    @attr('auth')
+    def auth_shortcut_test(self):
+        '''Test using the <SHORTCUT> auth format'''
+        cik = self.client.parentcik
+        rid = self.client.rid
+        shortcut = '{0}:c{1}'.format(cik, rid)
+
+        with tempfile.NamedTemporaryFile('w+') as cfgfile:
+            cfgfile.write("keys:\n")
+            cfgfile.write("    testme: {0}".format(shortcut))
+            cfgfile.flush()
+            cfgs = "--config={0}".format(cfgfile.name)
+            auth = 'testme'
+            isok = rpc(cfgs, 'info', auth)
+            self.ok(isok, 'info with <SHORTCUT>')
+
+
     def _readBack(self, res, limit):
         r = rpc('read',
                 res.parentcik,
@@ -1048,6 +1102,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
             keys = list(info.keys())
             self.assertTrue(len(keys) == len(allkeys) - 1 and k not in keys)
 
+    @attr('read')
     def read_test(self):
         '''Read command'''
         # record a large amount of data to a float datasource
@@ -1092,6 +1147,17 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         r = rpc('read', cik, rid1, '--start=44', '--timeformat=unix', '--limit=2')
         self.ok(r, '--end has a default', match='[0-9]+,12.36')
 
+    @attr('read')
+    def read_selection_test(self):
+        '''Read --selection option'''
+        cik = self.client.cik()
+        rid = self._createMultiple(cik, [
+            Resource(cik, 'dataport', {'format': 'string', 'name': 'string_port'})])[0]
+        r = rpc('record', cik, rid, '--value=1,a', '--value=2,a', '--value=3,c', '--value=4,d', '--value=5,e', '--value=8,f')
+        self.ok(r, 'record values')
+        r = rpc('read', cik, rid, '--start=1', '--end=10', '--timeformat=unix', '--limit=2', '--selection=givenwindow')
+        self.ok(r, 'read with --selection set', match=r'8,f\r?\n1,a')
+
     def utf8_test(self):
         '''Read a string with UTF8 characters'''
         cik = self.client.cik()
@@ -1358,7 +1424,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
 
     @attr('spec')
     def spec_url_test(self):
-        '''Test passing urls for spec file and spec scripts'''
+        '''Pass urls for spec file and spec scripts'''
         cik = self.client.cik()
 
         def spec_and_check(spec, aliases):
@@ -1381,6 +1447,9 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         spec_and_check(
             'https://raw.githubusercontent.com/exosite/exoline/master/test/files/spec_script_relative_url.yaml',
             ['temp_f', 'temp_c', 'convert.lua'])
+        spec_and_check(
+            'http://goo.gl/RQzS7d', # spec_script_relative_url.yaml again, this time via redirect
+            ['temp_f', 'temp_c', 'convert.lua'])
         # mainly this is testing embedded scripts
         spec_and_check(
             'https://raw.githubusercontent.com/exosite/exoline/master/test/files/spec_script_embedded.yaml',
@@ -1851,6 +1920,9 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         '''--discreet option'''
         cik = self.client.cik()
         r = rpc('--discreet', 'tree', cik)
+        # this is perhaps masking an issue.
+        if sys.version_info < (3, 0):
+            r.stdout = r.stdout.decode('utf-8')
         self.ok(r, search='cik: ' + cik[:20] + '01234567890123456789')
 
     def _createModel(self):
@@ -2126,9 +2198,9 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         childcik = r.stdout
 
         ridFloat, ridString, ridInteger, ridScript = self._createMultiple(childcik, [
-            Resource(cik, 'dataport', {'format': 'float', 'name': 'float_port', 'alias': 'float_alias'}),
-            Resource(cik, 'dataport', {'format': 'string', 'name': 'string_port', 'alias': 'string_alias'}),
-            Resource(cik, 'dataport', {'format': 'integer', 'name': 'integer_port', 'alias': 'int3ger_alias'}),
+            Resource(cik, 'dataport', {'format': 'float', 'name': 'float_port'}, alias='float_alias'),
+            Resource(cik, 'dataport', {'format': 'string', 'name': 'string_port'}, alias='string_alias'),
+            Resource(cik, 'dataport', {'format': 'integer', 'name': 'integer_port'}, alias='int3ger_alias'),
             Resource(cik, 'datarule', {
                     'format': 'string',
                     'name': 'script_port',
@@ -2141,7 +2213,7 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
                         'count': 'infinity',
                         'duration': 'infinity'
                     }
-                })])
+                }, alias='script_alias')])
 
         # set up a device clone with a serial number
         model = 'exolinetestmodel' + ''.join(random.choice(string.digits) for _ in range(5))
@@ -2196,6 +2268,89 @@ Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         self.l(r.stdout)
         self.assertEqual(len(r.stdout.split('\n')), 2, 'two matches: client model and clone')
 
+    def dump_test(self):
+        '''Dump command'''
+        cik = self.client.cik()
+        childrids = self._createMultiple(cik, [
+            Resource(cik, 'client', {'name': '你好' + str(i), 'alias': '你' + str(i)}) for i in range(10)])
+
+        # add some more things to one of the children
+        childrid = childrids[4]
+        r = rpc('info', cik, childrid, '--cikonly')
+        self.ok(r, 'look up one of the clients', match=self.RE_RID)
+        childcik = r.stdout
+
+        ridClient, ridFloat, ridString, ridInteger, ridScript = self._createMultiple(childcik, [
+            Resource(cik, 'client', {'name': '好'}),
+            Resource(cik, 'dataport', {'format': 'float', 'name': 'float_port'}, alias='float_alias'),
+            Resource(cik, 'dataport', {'format': 'string', 'name': 'string_port'}, alias='string_alias'),
+            Resource(cik, 'dataport', {'format': 'integer', 'name': 'integer_port'}, alias='int3ger_alias'),
+            Resource(cik, 'datarule', {
+                    'format': 'string',
+                    'name': 'script_port',
+                    'preprocess': [],
+                    'rule': {
+                        'script': 'local i = 0'
+                    },
+                    'visibility': 'parent',
+                    'retention': {
+                        'count': 'infinity',
+                        'duration': 'infinity'
+                    }
+                }, alias='script_alias')])
+
+        startts = 1418831000
+        valsFloat = [[i, 3.14159] for i in range(startts, startts + 250)]
+        r = rpc('record', childcik, ridFloat, *['--value={0},{1}'.format(t, v) for t, v in valsFloat])
+        self.ok(r, 'record floats')
+        valsString = [[i, '你好'] for i in range(startts, startts + 10)]
+        r = rpc('record', childcik, ridString, *['--value={0},{1}'.format(t, v) for t, v in valsString])
+        self.ok(r, 'record string')
+        valsInteger = [[i, 42] for i in range(startts, startts + 10)]
+        r = rpc('record', childcik, ridInteger, *['--value={0},{1}'.format(t, v) for t, v in valsInteger])
+        self.ok(r, 'record integer')
+
+        dumpfile = 'testdump.zip'
+        r = rpc('dump', cik, dumpfile)
+        self.ok(r, 'dump')
+
+        def extract_zip(input_zip):
+            input_zip = zipfile.ZipFile(input_zip)
+            l = [(name, json.loads(input_zip.read(name).decode("utf-8"))) for name in input_zip.namelist()]
+            return dict(l)
+
+        dumpzip = extract_zip(dumpfile)
+        it = dumpzip['infotree.json']
+        self.assertEqual(it['info']['key'], cik, 'key matches')
+
+        def findChild(infotree, rid):
+            for r in infotree['info']['children']:
+                if r['rid'] == rid:
+                    return r
+            return None
+
+        def testChildResource(infotree, rid, name, vals=None, alias=None):
+            r = findChild(infotree, rid)
+            self.assertNotEqual(r, None, 'find child ' + name)
+            # test name
+            self.assertEqual(r['info']['description']['name'], name, 'name matches')
+            # test alias
+            if alias is not None:
+                self.assertTrue(type(infotree['info']['aliases']) is dict, 'aliases is dict')
+                self.assertTrue(alias in infotree['info']['aliases'][rid], 'alias ' + alias + ' is found')
+            # test datapoints
+            if vals is not None:
+                dumpVals = dumpzip[r['info']['basic']['type'] + '.' + rid + '.json']
+                self.assertEqual(vals, dumpVals, 'dumped data points match for ' + name)
+
+        self.assertEqual(it['rid'], self.client.rid, 'root rid')
+        testChildResource(it, rid=childrid, name='你好4')
+        childit = findChild(it, childrid)
+        testChildResource(childit, rid=ridFloat, name='float_port', vals=valsFloat, alias='float_alias')
+        testChildResource(childit, rid=ridString, name='string_port', vals=valsString, alias='string_alias')
+        testChildResource(childit, rid=ridInteger, name='integer_port', vals=valsInteger, alias='int3ger_alias')
+        testChildResource(childit, rid=ridScript, name='script_port', vals=[])
+
 
 def tearDownModule(self):
     '''Do final things'''
